#code date 05/20/23
#refers to rev3 or rev4 pico board
#switched SCK and IO on dotstart LEDs

import time
import busio
import board
import rotaryio
import digitalio
from rainbowio import colorwheel
import adafruit_dotstar
import adafruit_ticks
from adafruit_debouncer import Debouncer
from digitalio import DigitalInOut, Direction, Pull
import supervisor
import random
import tupleColors
import adafruit_24lc32
#import os
#import storage


_TICKS_PERIOD = const(1 << 29)
_TICKS_MAX = const(_TICKS_PERIOD - 1)
_TICKS_HALFPERIOD = const(_TICKS_PERIOD // 2)

def ticks_add(ticks, delta):
    # "Add a delta to a base number of ticks, performing wraparound at 2**29ms."
    return (a + b) % _TICKS_PERIOD


def ticks_diff(ticks1, ticks2):
    # "Compute the signed difference between two ticks values, assuming that they are?
    # ??within 2**28 ticks"
    diff = (ticks1 - ticks2) & _TICKS_MAX
    diff = ((diff + _TICKS_HALFPERIOD) & _TICKS_MAX) - _TICKS_HALFPERIOD
    return diff


def ticks_less(ticks1, ticks2):
    # "Return true iff ticks1 is less than ticks2, assuming that they are within?
    # ??2**28 ticks"
    return ticks_diff(ticks1, ticks2) < 0

DEBUG = False
eepromOK = True

try:
    i2c = busio.I2C(board.GP5, board.GP4)
    eeprom = adafruit_24lc32.EEPROM_I2C(i2c)
except:
    eepromOK = False
    print("EEPROM Access Failed")

EEAnimationMode = 0
EEAnimationBrightness = 1
EEAnimationSpeed = 2
EEStaticWheelPos = 3
EESleepMode = 4

#menu modes
menuState = 0
MENUMAX = 4

MENURUN = 0
MENUBRIGHTNESS = 1
MENUANIMATION = 2
MENUANIMATIONCOLOR = 3
#sleep unused at this time, work on future
MENUSLEEPSELECT = 7
MENUSTOP = MENUMAX

#animation modes
animationMode = 0

DISCO = 0
FLAME = 1
FULLFADE = 2
FADEINOUT = 3
STATIC = 4

#sleep modes
sleepMode = 4
SLEEP15 = 0
SLEEP30 = 1
SLEEP60 = 2
SLEEP90 = 3
NOSLEEP = 4

BRIGHTMIN = 20
BRIGHTMAX = 95
BRIGHTDEF = 90
animationBrightness = BRIGHTDEF

SPEEDMIN = 150
SPEEDMAX = 20
SPEEDDEFAULT = 50
animationSpeed = SPEEDDEFAULT
animationSpeedAdder = 0

MAXWHEEL = 255
staticWheelPos = MAXWHEEL

lastUpdateTime = 0
currentTime = 0

selectEncoder = rotaryio.IncrementalEncoder(board.GP7, board.GP6)
selectEncoder_last_position = selectEncoder.position

selectButton = DigitalInOut(board.GP1)
selectButton.direction = Direction.INPUT
selectButton.pull = Pull.UP
selectPress = Debouncer(selectButton, 0.05)
selectCounter = 0
SELECTMAX = 5

num_pixels = 5
pixels = adafruit_dotstar.DotStar(
    board.GP2, board.GP3, num_pixels, brightness=BRIGHTDEF, auto_write=False
)

def color_fill(color, wait):
    pixels.fill(color)
    pixels.show()
    time.sleep(wait)

def wheel(pos):
    # Input a value 0 to 255 to get a color value.
    # The colours are a transition r - g - b - back to r.
    if pos < 0 or pos > 255:
        return 0, 0, 0
    if pos < 85:
        return int(255 - pos*3), int(pos*3), 0
    if pos < 170:
        pos -= 85
        return 0, int(255 - pos*3), int(pos*3)
    pos -= 170
    return int(pos * 3), 0, int(255 - (pos*3))

def getRandomColor():
    return (wheel(random.randint(0, 255)))

def disco():
    global pixelPosition
    if pixelPosition < 4:
        pixelPosition += 1
    else:
        pixelPosition = 0
    #pixels.fill(tupleColors.BLACK)
    pixels[pixelPosition % 5] = tupleColors.RED
    pixels[(pixelPosition+1) % 5] = tupleColors.ORANGE
    pixels[(pixelPosition+2) % 5] = tupleColors.GREEN
    pixels[(pixelPosition+3) % 5] = tupleColors.BLUE
    pixels[(pixelPosition+4) % 5] = tupleColors.PURPLE

def flame():
    global staticWheelPos
    global animationBrightness
    flameRange = 3
    for i in range(0, 5):
        #tempWheel = staticWheelPos -5 + random.randint(0,10)
        tempWheel = staticWheelPos - flameRange + random.randint(0,flameRange*2)
        if (tempWheel < 0):
            tempWheel += 255
        if (tempWheel > 255):
            tempWheel -= 255
        pixels[i] = wheel(tempWheel)
    tempBrightness = random.randint(0,50)
    if (animationBrightness - tempBrightness < 10):
        tempBrightness = 10
    else:
        tempBrightness = animationBrightness - tempBrightness

    #tempBrightness = animationBrightness / (random.randint(1,4))
    #if (tempBrightness < BRIGHTMIN):
        #tempBrightness = BRIGHTMIN
    #print("Brightness: ",tempBrightness)
    pixels.brightness = float(tempBrightness)/100

def fullFade():
    global fadeWheelPos
    if (fadeWheelPos < MAXWHEEL):
        fadeWheelPos += 1
    else:
        fadeWheelPos = 0

    pixels.fill(wheel(fadeWheelPos))

def fadeInOut():
    global fadeBrightness
    global fadeIncreasing
    global fadeWheelPos

    if (fadeIncreasing):
        if (fadeBrightness < animationBrightness):
            fadeBrightness += 5
        else:
            fadeIncreasing = False
    else:
        if (fadeBrightness > 10):
            fadeBrightness -= 5
        else:
            fadeIncreasing = True
            fadeWheelPos += random.randint(20,125)
            if (fadeWheelPos > 255):
                fadeWheelPos = fadeWheelPos - 255
    pixels.brightness = float(fadeBrightness)/100
    pixels.fill(wheel(fadeWheelPos))

def updateEEPROM():
    tempInt = int.from_bytes(eeprom[EEAnimationMode], "big")
    if (tempInt != animationMode):
        eeprom[EEAnimationMode] = animationMode
        print("Updated animation mode")
    tempInt = int.from_bytes(eeprom[EEAnimationBrightness], "big")
    if (tempInt != animationBrightness):
        eeprom[EEAnimationBrightness] = animationBrightness
        print("Updated animation brightness")
    tempInt = int.from_bytes(eeprom[EEAnimationSpeed], "big")
    if (tempInt != animationSpeed):
        eeprom[EEAnimationSpeed] = animationSpeed
        print("Updated animation speed")
    tempInt = int.from_bytes(eeprom[EEStaticWheelPos], "big")
    if (tempInt != staticWheelPos):
        eeprom[EEStaticWheelPos] = staticWheelPos
        print("Updated chosen color")
    tempInt = int.from_bytes(eeprom[EESleepMode], "big")
    if (tempInt != sleepMode):
        eeprom[EESleepMode] = sleepMode
        print("Updated sleep time")

def readEEPROM():
    global animationMode
    global animationSpeed
    global staticWheelPos
    global animationBrightness
    global sleepMode

    tempInt = int.from_bytes(eeprom[EEAnimationMode], "big")
    if (tempInt > 4):
        tempInt = 4
    if (tempInt < 0):
        tempInt = 0
    eeprom[EEAnimationMode] = tempInt
    animationMode = int.from_bytes(eeprom[EEAnimationMode], "big")
    print("Animation Mode: ",animationMode)

    tempInt = int.from_bytes(eeprom[EEAnimationSpeed], "big")
    if (tempInt > SPEEDMIN):
        tempInt = SPEEDMIN
    if (tempInt < SPEEDMAX):
        tempInt = SPEEDMAX
    eeprom[EEAnimationSpeed] = tempInt
    animationSpeed = int.from_bytes(eeprom[EEAnimationSpeed], "big")
    print("Animation Speed: ",animationSpeed)

    tempInt = int.from_bytes(eeprom[EEStaticWheelPos], "big")
    if (tempInt > MAXWHEEL):
        tempInt = MAXWHEEL
    if (tempInt < 0):
        tempInt = 0
    eeprom[EEStaticWheelPos] = tempInt
    staticWheelPos = int.from_bytes(eeprom[EEStaticWheelPos], "big")
    print("Color Wheel: ",staticWheelPos)

    tempInt = int.from_bytes(eeprom[EEAnimationBrightness], "big")
    if (tempInt > BRIGHTMAX):
        tempInt = BRIGHTMAX
    if  (tempInt < BRIGHTMIN):
        tempInt = BRIGHTMIN
    eeprom[EEAnimationBrightness] = tempInt
    animationBrightness = int.from_bytes(eeprom[EEAnimationBrightness], "big")
    print("Animation Brightness: ",animationBrightness)

    tempInt = int.from_bytes(eeprom[EESleepMode], "big")
    if (tempInt > 4):
        tempInt = 4
    if (tempInt < 0):
        tempInt = 0
    eeprom[EESleepMode] = tempInt
    sleepMode = int.from_bytes(eeprom[EESleepMode], "big")
    print("Sleep Setting: ",sleepMode)

oldColorWheel = [
    tupleColors.RED,
    tupleColors.YELLOW,
    tupleColors.ORANGE,
    tupleColors.GREEN,
    tupleColors.TEAL,
    tupleColors.CYAN,
    tupleColors.BLUE,
    tupleColors.PURPLE,
    tupleColors.MAGENTA,
    tupleColors.WHITE,
]
oldColorIndex = 0
colorMode = 0

# Increase or decrease this to speed up or slow down the animation.
# slice_alternating(0.1)

# Increase or decrease this to speed up or slow down the animation.
# slice_rainbow(0.1)

# time.sleep(0.5)

# Increase this number to slow down the rainbow animation.
# rainbow_cycle(0)
#color_fill(oldColorWheel[oldColorIndex], 0)
#pixels.show()

selectIsPressed = False
encoderDecrease = 0
encoderIncrease = 0
fullColorIndex = 0
doAnimation = False
fadeBrightness = BRIGHTMIN
fadeIncreasing = True

#with open("/test.txt", "a") as fp:
#f = open('test.txt')
    #print(fp.read())
    #fp.write("Test Value")
    #fp.flush()
#print("after")

pixelPosition = 0
fadeWheelPos = 0
lastUpdateTime = supervisor.ticks_ms()
currentTime = supervisor.ticks_ms()
if (eepromOK):
    readEEPROM()

while True:
    #selectPress.update
    if ticks_diff(currentTime, lastUpdateTime) > (animationSpeed + animationSpeedAdder):
        lastUpdateTime = supervisor.ticks_ms()
        doAnimation = True
    else:
        currentTime = supervisor.ticks_ms()

    current_position = selectEncoder.position
    position_change = current_position - selectEncoder_last_position
    position_change = current_position - selectEncoder_last_position

    if position_change > 0:
        encoderIncrease = 1
        if (DEBUG):
            print("Encoder Increase: ", oldColorIndex)
    elif position_change < 0:
        encoderDecrease = 1
        if (DEBUG):
            print("Encoder Decrease: ", oldColorIndex)
    selectEncoder_last_position = current_position

    if not selectButton.value and not selectIsPressed:
        if menuState < MENUMAX:
            menuState += 1
        else:
            menuState = 0
            if (eepromOK):
                updateEEPROM()
        selectIsPressed = True
        encoderIncrease = 0
        encoderDecrease = 0
        pixels.brightness = float(animationBrightness)/100
        if (DEBUG):
            print("Menu State: ", menuState)
    if selectButton.value:
        selectIsPressed = False

###--- Menu State Run ---###
    if menuState == MENURUN:
        if (doAnimation):
            if animationMode == DISCO:
                disco()
                animationSpeedAdder = 50
            elif animationMode == FLAME:
                flame()
                animationSpeedAdder = random.randint(20,125)
            elif animationMode == FULLFADE:
                fullFade()
                animationSpeedAdder = 0
            elif animationMode == FADEINOUT:
                fadeInOut()
                animationSpeedAdder = 0
            elif animationMode == STATIC:
                pixels.fill(wheel(staticWheelPos))
            doAnimation = False

        if encoderIncrease == 1:
            if ((animationSpeed < SPEEDMIN) and not (animationMode == STATIC)):
                animationSpeed += 10
            encoderIncrease = 0
            if (DEBUG):
                print("Speed Decrease: ", animationSpeed)
        if encoderDecrease == 1:
            if ((animationSpeed > SPEEDMAX) and not (animationMode == STATIC)):
                animationSpeed -= 10
            encoderDecrease = 0
            if (DEBUG):
                print("Speed Increase: ", animationSpeed)

###--- Menu State Brightness Select ---###
    elif menuState == MENUBRIGHTNESS:
        pixels[0] = tupleColors.BLUE
        pixels[1] = tupleColors.RED
        pixels[2] = tupleColors.GREEN
        pixels[3] = tupleColors.PURPLE
        pixels[4] = tupleColors.WHITE

        if encoderIncrease == 1:
            if (animationBrightness < BRIGHTMAX):
                animationBrightness += 5
            encoderIncrease = 0
            pixels.brightness = float(animationBrightness)/100
            if (DEBUG):
                print("Brightness Increase: ", animationBrightness)
        if encoderDecrease == 1:
            if (animationBrightness > BRIGHTMIN):
                animationBrightness -= 5
            encoderDecrease = 0
            pixels.brightness = float(animationBrightness)/100
            if (DEBUG):
                print("Brightness Decrease: ", animationBrightness)

###--- Menu State Animation Select ---###
    elif menuState == MENUANIMATION:
        if encoderIncrease == 1:
            if animationMode < 4:
                animationMode += 1
            else:
                animationMode = 0
            encoderIncrease = 0
        if encoderDecrease == 1:
            if animationMode > 0:
                animationMode -= 1
            else:
                animationMode = 4
            encoderDecrease = 0

        pixels.fill(tupleColors.BLACK)
        pixels[animationMode] = tupleColors.RED

###--- Menu State Choose Color ---###
    elif menuState == MENUANIMATIONCOLOR:
        if encoderIncrease == 1:
            #if fullColorIndex < len(tupleColors.fullColorWheel) -1:
                #fullColorIndex += 1
            #else:
                #fullColorIndex = 0
            if (staticWheelPos < MAXWHEEL-5):
                staticWheelPos += 5
            else:
                staticWheelPos = 0
            encoderIncrease = 0
            if (DEBUG):
                print("Color Index: ",staticWheelPos)
        if encoderDecrease == 1:
            if (staticWheelPos >= 5):
                staticWheelPos -= 5
            else:
                staticWheelPos = MAXWHEEL-5
            #if fullColorIndex > 0:
                #fullColorIndex -= 1
            #else:
                #fullColorIndex = len(tupleColors.fullColorWheel) - 1
            encoderDecrease = 0
            if (DEBUG):
                print("Color Index: ",staticWheelPos)
        pixels.fill(wheel(staticWheelPos))
        pixels[1] = tupleColors.BLACK

###--- Menu State Sleep Modes ---###
    elif menuState == MENUSLEEPSELECT:
        if encoderIncrease == 1:
            if sleepMode < 4:
                sleepMode += 1
            #else:
                #sleepMode = 0
            encoderIncrease = 0
            print("Sleep Setting: ",sleepMode)
        if encoderDecrease == 1:
            if sleepMode > 0:
                sleepMode -= 1
            #else:
                #sleepMode = 4
            encoderDecrease = 0
            print("Sleep Setting: ",sleepMode)
        pixels.fill(tupleColors.BLACK)
        for x in range(sleepMode+1):
            pixels[x] = tupleColors.PURPLE

###--- Menu State Stop - no animation, black colors ---###
    elif menuState == MENUSTOP:
        pixels.fill(tupleColors.BLACK)

    pixels.show()
    #time.sleep(0.005)  # debounce
